<?php
/**
 * @package        Joomla
 * @subpackage     Events Booking
 * @author         Tuan Pham Ngoc
 * @copyright      Copyright (C) 2012 - 2021 Ossolution Team
 * @license        GNU/GPL, see LICENSE.php
 */

defined('_JEXEC') or die;

use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Uri\Uri;
use Joomla\Registry\Registry;
use Mpdf\Config\FontVariables;
use Mpdf\Mpdf;

class plgEventbookingMPDF extends CMSPlugin
{
	/**
	 * Application object.
	 *
	 * @var    \Joomla\CMS\Application\CMSApplication
	 */
	protected $app;

	/**
	 * Generate PDF file
	 *
	 * @param   array   $pages  '
	 * @param   string  $filePath
	 * @param   array   $options
	 *
	 * @return bool
	 * @throws \Mpdf\MpdfException
	 */
	public function onEBBeforeGeneratePDF($pages, $filePath, $options)
	{
		$pdf = $this->getPDF($options);

		foreach ($pages as $page)
		{
			$pdf->AddPage();

			if (!empty($page->options))
			{
				$pageOptions = $page->options;
			}
			else
			{
				$pageOptions = $options;
			}

			if (!empty($pageOptions['bg_image']))
			{
				$this->setPageBackgroundImage($pdf, $pageOptions);
			}

			$pageContent = EventbookingHelper::convertImgTags($page->content);

			$pageContent = $this->getHtml($pageContent, $options);

			$pdf->WriteHTML($pageContent);
		}

		$pdf->Output($filePath);

		// Need to return true to tell Events Booking that the PDF file was successfully generated
		return true;
	}

	/**
	 * Get MPDF object
	 *
	 * @param   array  $options
	 *
	 * @return \Mpdf\Mpdf
	 */
	private function getPDF($options)
	{
		require_once __DIR__ . '/mpdf/vendor/autoload.php';

		$config = EventbookingHelper::getConfig();

		$options    = new Registry($options);
		$pdfOptions = [
			'format' => $options->get('PDF_PAGE_FORMAT', 'A4') . '-' . $options->get('PDF_PAGE_ORIENTATION', 'P'),
		];

		$options->def('pdf_margin_left', $config->get('pdf_margin_left'));
		$options->def('pdf_margin_top', $config->get('pdf_margin_top'));
		$options->def('pdf_margin_right', $config->get('pdf_margin_right'));

		if ($options->get('pdf_margin_left'))
		{
			$pdfOptions['margin_left'] = $options->get('pdf_margin_left');
		}

		if ($options->get('pdf_margin_top'))
		{
			$pdfOptions['margin_top'] = $options->get('pdf_margin_top');
		}

		if ($options->get('pdf_margin_right'))
		{
			$pdfOptions['margin_right'] = $options->get('pdf_margin_right');
		}

		if (!Folder::exists(JPATH_ROOT . '/plugins/eventbooking/mpdf/mpdf/ttfonts'))
		{
			$pdfOptions['mode'] = 'c';
		}

		if (File::exists(__DIR__ . '/config.php'))
		{
			$customConfig = require __DIR__ . '/config.php';
		}
		else
		{
			$customConfig = [];
		}

		if (isset($customConfig['fontdata']))
		{
			$defaultFontConfig      = (new FontVariables())->getDefaults();
			$fontData               = $defaultFontConfig['fontdata'];
			$pdfOptions['fontdata'] = $fontData + $customConfig['fontdata'];
		}

		$customConfigKeys = [
			'default_font',
			'default_font_size',
			'margin_left',
			'margin_top',
			'margin_right',
			'margin_bottom',
			'margin_footer',
		];

		foreach ($customConfigKeys as $key)
		{
			if (isset($customConfig[$key]))
			{
				$pdfOptions[$key] = $customConfig[$key];
			}
		}

		$mpdf = new Mpdf($pdfOptions);

		$mpdf->SetTitle($options->get('title'));
		$mpdf->SetCreator($this->app->get('sitename'));

		return $mpdf;
	}

	/**
	 * Method to set background image for a PDF page
	 *
	 * @param   \Mpdf\Mpdf  $mpdf
	 *
	 * @param   array       $options
	 */
	private function setPageBackgroundImage($mpdf, $options)
	{
		$options = new Registry($options);

		// Handle background image
		if ($options->get('bg_image'))
		{
			$backgroundImage = $options->get('bg_image');

			$mpdf->SetDefaultBodyCSS('background', "url('" . $backgroundImage . "')");
			$mpdf->SetDefaultBodyCSS('background-image-resize', 6);


//			$backgroundLeft   = $options->get('bg_left', '');
//			$backgroundTop    = $options->get('bg_top', '');
//			$backgroundWidth  = $options->get('bg_width', 0);
//			$backgroundHeight = $options->get('bg_height', 0);
//
//			$mpdf->Image($backgroundImage, $backgroundLeft, $backgroundTop, $backgroundWidth, $backgroundHeight, File::getExt($backgroundImage));
		}
	}

	/**
	 * Get HTML code for pdf page
	 *
	 * @param   string  $page
	 *
	 * @return string
	 * @var array       $options
	 *
	 */
	private function getHtml($page, $options = [])
	{
		$options            = new Registry($options);
		$config             = EventbookingHelper::getConfig();
		$css                = trim($config->pdf_css);
		$url                = Uri::root();
		$commonCSSFileUrl   = '';
		$specificCssFileUrl = '';

		if (File::exists(JPATH_ROOT . '/media/com_eventbooking/assets/css/pdf.css'))
		{
			$commonCSSFileUrl = $url . 'media/com_eventbooking/assets/css/pdf.css';
		}
		elseif (File::exists(__DIR__ . '/css/pdf.css'))
		{
			$commonCSSFileUrl = $url . 'plugins/eventbooking/mpdf/css/pdf.css';
		}

		$type = $options->get('type', '');

		if ($type && File::exists(__DIR__ . '/media/com_eventbooking/assets/css/' . $type . '.css'))
		{
			$specificCssFileUrl = $url . 'media/com_eventbooking/assets/css/' . $type . '.css';
		}
		elseif ($type && File::exists(__DIR__ . '/css/' . $type . '.css'))
		{
			$specificCssFileUrl = $url . 'plugins/eventbooking/mpdf/css/' . $type . '.css';
		}

		$html   = [];
		$html[] = '<html>';

		if ($css || $commonCSSFileUrl || $specificCssFileUrl)
		{
			$html[] = '	<head>';

			if ($css)
			{
				$html[] = '		<style>';
				$html[] = '		    ' . $css;
				$html[] = '		</style>';
			}

			if ($commonCSSFileUrl)
			{
				$html[] = '     <link rel="stylesheet" href="' . $commonCSSFileUrl . '">';
			}

			if ($specificCssFileUrl)
			{
				$html[] = '     <link rel="stylesheet" href="' . $specificCssFileUrl . '">';
			}

			$html[] = '	</head>';
		}

		if ($type)
		{
			$bodyCssClass = ' class="' . $type . '"';
		}
		else
		{
			$bodyCssClass = '';
		}

		$html[] = '<body' . $bodyCssClass . '>';
		$html[] = ' ' . $page;
		$html[] = '</body>';

		$html[] = '</html>';

		return implode("\r\n", $html);
	}
}
